@@ -17,9 +17,10 @@ using namespace apolloron;
 
 const char *INKF_PROGNAME = "inkf";
 const char *INKF_BRIEF    = "Yet Another Network Kanji Filter";
-const char *INKF_VERSION  = "0.9.5";
+const char *INKF_VERSION  = "0.9.8";
 const char *INKF_COPY     = "Copyright (C) 2002-2009 Tachibanamasashi";
 const char *INKF_DEF_OUT  = "inkf.out";
+const int REDIRECT_MAX = 10;
 
 typedef enum {
   MIME_NONE,
@@ -67,7 +68,7 @@ typedef struct {
 } TOption;
 
 
-TOption *set_option(TOption *option, int argc, const char **argv);
+bool set_option(TOption &option, int argc, const char **argv);
 void guess(const String &str, const char *input_charset, FILE *fpout);
 void convert(String &str, const TOption &option, FILE *fpout);
 void show_help();
@@ -84,7 +85,7 @@ int main(int argc, char *argv[]) {
   int i, j, k, l;
 
   // set option
-  if (set_option(&option, argc, (const char **)argv) == (TOption *)NULL) {
+  if (!set_option(option, argc, (const char **)argv)) {
     fprintf(stderr, "Invalid parameter(s).\n");
     retval = -1;
     goto main_exit;
@@ -155,15 +156,25 @@ int main(int argc, char *argv[]) {
       }
     }
   } else {
+    bool charset_autodetect_pre;
+    charset_autodetect_pre = false;
+
+    if (!strcasecmp(option.input_charset, "AUTODETECT")) {
+      charset_autodetect_pre = true;
+    }
 
     // read from file(s) or specified URL
     for (i = 0; option.input_filenames[i] != '\0'; i++) {
+        if (charset_autodetect_pre) {
+          strcpy(option.input_charset, "AUTODETECT");
+	}
 #if __OPENSSL == 1
       if (!strncasecmp(option.input_filenames[i], "http://", 7) ||
           !strncasecmp(option.input_filenames[i], "https://", 8)) {
 #else
       if (!strncasecmp(option.input_filenames[i], "http://", 7)) {
 #endif
+        char *url;
         Socket tmp_socket;
         char host[1024], port[20], path[1024], port_tmp[20];
         char *p;
@@ -173,8 +184,15 @@ int main(int argc, char *argv[]) {
         bool header_end;
         bool ischunked;
         long chunk_size;
+        int redirect_count;
+
+        redirect_count = 0;
+        url = new char [strlen(option.input_filenames[i]) + 1];
+        strcpy(url, option.input_filenames[i]);
+
+HTTP_START:
 #if __OPENSSL == 1
-        if (!strncasecmp(option.input_filenames[i], "http://", 7)) {
+        if (!strncasecmp(url, "http://", 7)) {
 #endif
           strcpy(port, "80");
           host_base = 7;
@@ -186,7 +204,7 @@ int main(int argc, char *argv[]) {
           ssl = true;
         }
 #endif
-        strncpy(host, option.input_filenames[i] + host_base, 1023);
+        strncpy(host, url + host_base, 1023);
         host[1023] = '\0';
         p = strchr(host, '/');
         if (p != NULL) {
@@ -196,7 +214,8 @@ int main(int argc, char *argv[]) {
         if (p != NULL) {
           *p = '\0';
         }
-        p = option.input_filenames[i] + host_base + strlen(host);
+        port_tmp[0] = '\0';
+        p = url + host_base + strlen(host);
         if (*p == ':') {
           for (j = 0; p[1 + j] != '\0' && j < 19; j++) {
             if (p[1 + 1] == '/') {
@@ -209,11 +228,12 @@ int main(int argc, char *argv[]) {
         if (port_tmp[0] != '\0') {
           strcpy(port, port_tmp);
         }
-        p = strchr(option.input_filenames[i] + host_base, '/');
+        p = strchr(url + host_base, '/');
         strncpy(path, p?p:"/", 1023);
         path[1023] = '\0';
+        delete [] url;
         charset_autodetect = false;
-        if (!strcasecmp(option.input_charset, "AUTODETECT")) {
+        if (charset_autodetect_pre) {
           charset_autodetect = true;
         }
         tmp_socket.setTimeout(5);
@@ -234,21 +254,69 @@ int main(int argc, char *argv[]) {
         header_end = false;
         ischunked = false;
         chunk_size = -1;
-        while (tmp_socket.error() == 0 && 0 < tmp_socket.receiveLine().len()) {
+        while (tmp_socket.error() == 0 && tmp_socket.connected()) {
           char *endptr;
 
+          tmp_socket.receiveLine();
+          if (tmp_socket.receivedData().len() == 0) {
+            break;
+          }
+
+          // redirect
+          if (!header_end &&
+              !strncasecmp(tmp_socket.receivedData().c_str(), "Location", 8) &&
+              8 <= tmp_socket.receivedData().search(":") &&
+              9 <= tmp_socket.receivedData().searchCase("http")) {
+            url = new char [tmp_socket.receivedData().len() + 1];
+            p = url;
+            p[0] = '\0';
+            j = 0;
+            for (j = 9; j < tmp_socket.receivedData().len(); j++) {
+              if (url[0] == '\0') {
+                if (!isspace(tmp_socket.receivedData()[j]) && tmp_socket.receivedData()[j] != ':') {
+                  *p = tmp_socket.receivedData()[j];
+                  p++;
+                }
+              } else if (!isspace(tmp_socket.receivedData()[j])) {
+                *p = tmp_socket.receivedData()[j];
+                p++;
+              }
+            }
+            *p = '\0';
+
+#if __OPENSSL == 1
+            if (!strncasecmp(url, "http://", 7) ||
+                !strncasecmp(url, "https://", 8)) {
+#else
+            if (!strncasecmp(url, "http://", 7)) {
+#endif
+              tmp_socket.clear();
+              redirect_count++;
+              if (redirect_count <= REDIRECT_MAX) {
+                goto HTTP_START;
+              }
+            }
+
+            delete [] url;
+          }
+
           if (!header_end &&
               !strncasecmp(tmp_socket.receivedData().c_str(), "Transfer-Encoding", 17) &&
+              17 <= tmp_socket.receivedData().search(":") &&
               18 <= tmp_socket.receivedData().searchCase("chunked")) {
             ischunked = true;
           }
 
           if (header_end && ischunked) {
+            if (tmp_socket.receivedData()[0] == '\n' || tmp_socket.receivedData() == "\r\n") {
+              continue;
+            }
             chunk_size = strtol(tmp_socket.receivedData().c_str(), &endptr, 16);
             if (chunk_size <= 0) {
               continue;
             }
             tmp_socket.receive(chunk_size);
+            tmp_socket.receivedData().useAsText();
           }
 
           if (charset_autodetect && 0 <= (l = tmp_socket.receivedData().searchCase("charset="))) {
@@ -328,169 +396,169 @@ main_exit:
 }
 
 
-TOption *set_option(TOption *option, int argc, const char **argv) {
+bool set_option(TOption &option, int argc, const char **argv) {
   bool o_param_found = false;
   int input_filenames_max = 0;
   int i, j;
 
   // set default values
-  option->flag_show_help = false;
-  option->flag_show_version = false;
-  option->flag_guess = false;
-  option->flag_overwrite = false;
-  option->flag_html_to_plain = false;
-  option->flag_hankaku_ascii = false;
-  option->flag_zenkaku_ascii = false;
-  option->flag_hankaku_katakana = false;
-  option->flag_zenkaku_katakana = false;
-  option->flag_hiragana = false;
-  option->flag_katakana = false;
-  option->flag_midi = false;
-  strcpy(option->input_charset, "AUTODETECT");
-  option->output_charset[0] = '\0';
-  option->mime_decode = MIME_NONE;
-  option->mime_encode = MIME_NONE;
-  option->line_mode = LINE_MODE_NOCONVERSION;
-  option->input_filenames = (char **)NULL; // default is stdin
-  option->output_filename = NULL; // default is stdout
+  option.flag_show_help = false;
+  option.flag_show_version = false;
+  option.flag_guess = false;
+  option.flag_overwrite = false;
+  option.flag_html_to_plain = false;
+  option.flag_hankaku_ascii = false;
+  option.flag_zenkaku_ascii = false;
+  option.flag_hankaku_katakana = false;
+  option.flag_zenkaku_katakana = false;
+  option.flag_hiragana = false;
+  option.flag_katakana = false;
+  option.flag_midi = false;
+  strcpy(option.input_charset, "AUTODETECT");
+  option.output_charset[0] = '\0';
+  option.mime_decode = MIME_NONE;
+  option.mime_encode = MIME_NONE;
+  option.line_mode = LINE_MODE_NOCONVERSION;
+  option.input_filenames = (char **)NULL; // default is stdin
+  option.output_filename = NULL; // default is stdout
 
   for (i = 1; i < argc; i++) {
     if (argv[i][0] == '-' && argv[i][1] == '-') {
       if (!strcasecmp(argv[i], "--help")) {
-        option->flag_show_help = true;
+        option.flag_show_help = true;
         break;
       }
       if (!strcasecmp(argv[i], "--version")) {
-        option->flag_show_version = true;
+        option.flag_show_version = true;
         break;
       }
       if (!strcasecmp(argv[i], "--guess")) {
-        option->flag_guess = true;
+        option.flag_guess = true;
       } else if (!strncasecmp(argv[i], "--ic=", 5)) {
-        strncpy(option->input_charset, argv[i]+5, 15);
-        option->input_charset[15] = '\0';
+        strncpy(option.input_charset, argv[i]+5, 15);
+        option.input_charset[15] = '\0';
       } else if (!strncasecmp(argv[i], "--oc=", 5)) {
-        strncpy(option->output_charset, argv[i]+5, 15);
-        option->output_charset[15] = '\0';
+        strncpy(option.output_charset, argv[i]+5, 15);
+        option.output_charset[15] = '\0';
       } else if (!strcasecmp(argv[i], "--overwrite")) {
-        option->flag_overwrite = true;
+        option.flag_overwrite = true;
       } else if (!strcasecmp(argv[i], "--html-to-plain")) {
-        option->flag_html_to_plain = true;
+        option.flag_html_to_plain = true;
       } else if (!strcasecmp(argv[i], "--hankaku-ascii")) {
-        option->flag_hankaku_ascii = true;
+        option.flag_hankaku_ascii = true;
       } else if (!strcasecmp(argv[i], "--zenkaku-ascii")) {
-        option->flag_zenkaku_ascii = true;
+        option.flag_zenkaku_ascii = true;
       } else if (!strcasecmp(argv[i], "--hankaku-katakana")) {
-        option->flag_hankaku_katakana = true;
+        option.flag_hankaku_katakana = true;
       } else if (!strcasecmp(argv[i], "--zenkaku-katakana")) {
-        option->flag_zenkaku_katakana = true;
+        option.flag_zenkaku_katakana = true;
       } else if (!strcasecmp(argv[i], "--hiragana")) {
-        option->flag_hiragana = true;
+        option.flag_hiragana = true;
       } else if (!strcasecmp(argv[i], "--katakana")) {
-        option->flag_katakana = true;
+        option.flag_katakana = true;
       } else if (!strcasecmp(argv[i], "--midi")) {
-        option->flag_midi = true;
+        option.flag_midi = true;
       } else {
-        return (TOption *)NULL; // invalid parameter
+        return false; // invalid parameter
       }
     } else if (argv[i][0] == '-') {
       for (j = 1; argv[i][j] != '\0'; j++) {
         switch (argv[i][j]) {
           case 'v':
-            option->flag_show_version = true;
-            return option;
+            option.flag_show_version = true;
+            return true;
             break;
           case 'V':
-            option->flag_show_help = true;
-            return option;
+            option.flag_show_help = true;
+            return true;
             break;
           case 'g':
-            option->flag_guess = true;
+            option.flag_guess = true;
             break;
           case 'j':
-            strcpy(option->output_charset, "ISO-2022-JP");
+            strcpy(option.output_charset, "ISO-2022-JP");
             break;
           case 's':
-            strcpy(option->output_charset, "SHIFT_JIS");
+            strcpy(option.output_charset, "SHIFT_JIS");
             break;
           case 'e':
-            strcpy(option->output_charset, "EUC-JP");
+            strcpy(option.output_charset, "EUC-JP");
             break;
           case 'w':
-            strcpy(option->output_charset, "UTF-8");
+            strcpy(option.output_charset, "UTF-8");
             break;
           case 'k':
-            strcpy(option->output_charset, "EUC-KR");
+            strcpy(option.output_charset, "EUC-KR");
             break;
           case 'c':
-            strcpy(option->output_charset, "GB2312");
+            strcpy(option.output_charset, "GB2312");
             break;
           case 't':
-            strcpy(option->output_charset, "BIG5");
+            strcpy(option.output_charset, "BIG5");
             break;
           case 'J':
-            strcpy(option->input_charset, "ISO-2022-JP");
+            strcpy(option.input_charset, "ISO-2022-JP");
             break;
           case 'S':
-            strcpy(option->input_charset, "SHIFT_JIS");
+            strcpy(option.input_charset, "SHIFT_JIS");
             break;
           case 'E':
-            strcpy(option->input_charset, "EUC-JP");
+            strcpy(option.input_charset, "EUC-JP");
             break;
           case 'W':
-            strcpy(option->input_charset, "UTF-8");
+            strcpy(option.input_charset, "UTF-8");
             break;
           case 'K':
-            strcpy(option->input_charset, "EUC-KR");
+            strcpy(option.input_charset, "EUC-KR");
             break;
           case 'C':
-            strcpy(option->input_charset, "GB2312");
+            strcpy(option.input_charset, "GB2312");
             break;
           case 'T':
-            strcpy(option->input_charset, "BIG5");
+            strcpy(option.input_charset, "BIG5");
             break;
           case 'm':
-            option->mime_decode = MIME_AUTODETECT;
+            option.mime_decode = MIME_AUTODETECT;
             if (argv[i][j + 1] == 'B') {
-              option->mime_decode = MIME_BASE64;
+              option.mime_decode = MIME_BASE64;
               j++;
             } else if (argv[i][j + 1] == 'Q') {
-              option->mime_decode = MIME_QUOTED_PRINTABLE;
+              option.mime_decode = MIME_QUOTED_PRINTABLE;
               j++;
             } else if (argv[i][j + 1] == 'N') {
-              option->mime_decode = MIME_AUTODETECT;
+              option.mime_decode = MIME_AUTODETECT;
               j++;
             } else if (argv[i][j + 1] == '0') {
-              option->mime_decode = MIME_NONE;
+              option.mime_decode = MIME_NONE;
               j++;
             }
             break;
           case 'M':
-            option->mime_encode = MIME_BASE64_FULL;
+            option.mime_encode = MIME_BASE64_FULL;
             if (argv[i][j + 1] == 'B') {
-              option->mime_encode = MIME_BASE64;
+              option.mime_encode = MIME_BASE64;
               j++;
             } else if (argv[i][j + 1] == 'Q') {
-              option->mime_encode = MIME_QUOTED_PRINTABLE;
+              option.mime_encode = MIME_QUOTED_PRINTABLE;
               j++;
             }
             break;
           case 'Z':
-            option->flag_hankaku_ascii = true;
+            option.flag_hankaku_ascii = true;
             break;
           case 'O':
             o_param_found = true;
             break;
           case 'L':
-            option->line_mode = LINE_MODE_NOCONVERSION;
+            option.line_mode = LINE_MODE_NOCONVERSION;
             if (argv[i][j + 1] == 'u') {
-              option->line_mode = LINE_MODE_LF;
+              option.line_mode = LINE_MODE_LF;
               j++;
             } else if (argv[i][j + 1] == 'w') {
-              option->line_mode = LINE_MODE_CRLF;
+              option.line_mode = LINE_MODE_CRLF;
               j++;
             } else if (argv[i][j + 1] == 'm') {
-              option->line_mode = LINE_MODE_CR;
+              option.line_mode = LINE_MODE_CR;
               j++;
             }
             break;
@@ -498,21 +566,21 @@ TOption *set_option(TOption *option, int argc, const char **argv) {
           case 'u': // ignore unbufferd option
             break;
           default:
-            return (TOption *)NULL; // invalid parameter
+            return false; // invalid parameter
             break;
         }
       }
     } else {
       j = argc - i - (o_param_found?1:0);
       if (0 < j) {
-        option->input_filenames = new char * [j + 1];
+        option.input_filenames = new char * [j + 1];
       }
       for (j = i; j < argc; j++) {
         if (o_param_found && j == argc - 1) {
-          option->output_filename = argv[j];
+          option.output_filename = argv[j];
         } else if (argv[j][0] != '\0') {
-          option->input_filenames[input_filenames_max] = (char *)argv[j];
-          option->input_filenames[input_filenames_max + 1] = NULL;
+          option.input_filenames[input_filenames_max] = (char *)argv[j];
+          option.input_filenames[input_filenames_max + 1] = NULL;
           input_filenames_max++;
         }
       }
@@ -521,20 +589,20 @@ TOption *set_option(TOption *option, int argc, const char **argv) {
     }
   }
 
-  if (input_filenames_max == 0 && option->input_filenames != (char **)NULL) {
-    delete [] option->input_filenames;
-    option->input_filenames = (char **)NULL;
+  if (input_filenames_max == 0 && option.input_filenames != (char **)NULL) {
+    delete [] option.input_filenames;
+    option.input_filenames = (char **)NULL;
   }
 
-  if (option->output_filename != NULL && option->output_filename[0] == '\0') {
-    option->output_filename = NULL;
+  if (option.output_filename != NULL && option.output_filename[0] == '\0') {
+    option.output_filename = NULL;
   }
 
-  if (o_param_found && option->output_filename == NULL) {
-    option->output_filename = INKF_DEF_OUT;
+  if (o_param_found && option.output_filename == NULL) {
+    option.output_filename = INKF_DEF_OUT;
   }
 
-  return option;
+  return true;
 }
 
 
