@@ -35,6 +35,10 @@ namespace {
 #define INADDR_NONE 0xFFFFFFFF
 #endif
 
+#ifndef MSG_WAITALL
+#define MSG_WAITALL 0x8
+#endif
+
 } // namespace
 
 
@@ -120,12 +124,18 @@ bool Socket::connect(const char *host, const char *port, bool ssl) {
 #endif
 
   (*this).nErrno = 0;
-  if ((*this).bConnected || port == NULL) {
+  if ((*this).bConnected) {
+    return false;
+  }
+
+  if (port == NULL || port[0] == '\0') {
+    (*this).nErrno = -1;
     return false;
   }
 
 #if __OPENSSL == 0
   if (ssl) {
+    (*this).nErrno = -1;
     return false;
   }
 #endif
@@ -158,7 +168,7 @@ bool Socket::connect(const char *host, const char *port, bool ssl) {
   }
 #endif
 
-  if (getaddrinfo(host, port, &hints, &res) != 0) {
+  if (getaddrinfo(destination, port, &hints, &res) != 0) {
     (*this).nErrno = -1;
     return false;
   }
@@ -192,11 +202,13 @@ bool Socket::connect(const char *host, const char *port, bool ssl) {
   if (addr != INADDR_NONE) {
     hp = gethostbyaddr((const char *)&addr , 4 , AF_INET);
     if (hp == NULL) {
+      (*this).nErrno = -1;
       return false;
     }
   } else {
     hp = gethostbyname(destination);
     if (hp == NULL) {
+      (*this).nErrno = -1;
       return false;
     }
   }
@@ -219,6 +231,7 @@ bool Socket::connect(const char *host, const char *port, bool ssl) {
       struct servent *se;
       se = getservbyname(port, "tcp");
       if (se == NULL) {
+        (*this).nErrno = -1;
         return false;
       }
       dstAddr.sin_port = se->s_port;
@@ -229,6 +242,7 @@ bool Socket::connect(const char *host, const char *port, bool ssl) {
 
   // create socket
   if(((*this).dstSocket = socket(AF_INET, SOCK_STREAM, 0)) < 0) {
+    (*this).nErrno = -1;
     return false;
   }
 
@@ -426,6 +440,7 @@ bool Socket::send(const String &str) {
 
   if (ret < 0) {
     (*this).nErrno = errno;
+    (*this).disconnect();
     return false;
   }
 
@@ -477,34 +492,49 @@ String& Socket::receive(long size) {
   fd_set readOK, mask;
   int width;
   long total;
-  char tmp[2];
-  int numrcv;
+  char *buf;
+  long numrcv;
 
   *((*this).tmpString) = "";
 
-  if ((*this).bConnected == false || (*this).dstSocket < 0 || size <= 0) {
+  if ((*this).bConnected == false || (*this).dstSocket < 0 || size < 0) {
     (*this).nErrno = -1;
     return *((*this).tmpString);
   }
 
   (*this).nErrno = 0;
+  if (size == 0) {
+    return *((*this).tmpString);
+  }
+
   total = 0;
   (*((*this).tmpString)).setFixedLength(size);
   (*((*this).tmpString)).useAsBinary(total);
 
 #if __OPENSSL == 1
   if ((*this).bSSLStream) {
+    buf = new char [size + 1];
+    total = 0;
     do {
-      numrcv = SSL_read((SSL *)((*this).pSSL), tmp, 1);
-      if (numrcv == 1) {
-        (*((*this).tmpString))[total] = tmp[0];
-        total++;
-      } else if (numrcv == 0) {
-        (*this).nErrno = -1;
+      numrcv = SSL_read((SSL *)((*this).pSSL), buf + total, size - total);
+      if (numrcv < 0) {
         break;
+      } else if (numrcv == 0 && total < size) {
+        usleep(1000);
+        continue;
       }
-    } while (total < size);
-    (*((*this).tmpString)).useAsBinary(total);
+      total += numrcv;
+    } while (0 < size - total);
+    buf[total] = '\0';
+    buf[size] = '\0';
+    if (0 <= total) {
+      (*((*this).tmpString)).setBinary(buf, total);
+    }
+    delete [] buf;
+    if (total < size) {
+      (*this).nErrno = errno;
+      (*this).disconnect();
+    }
     return *((*this).tmpString);
   }
 #endif
@@ -519,19 +549,30 @@ String& Socket::receive(long size) {
 
   if (0 < select(width, &readOK, NULL, NULL, &timeout)) {
     if (FD_ISSET((*this).dstSocket, &readOK)) {
+      buf = new char [size + 1];
+      total = 0;
       do {
-        numrcv = recv((*this).dstSocket, tmp, 1, 0);
-        if (0 < numrcv) {
-          (*((*this).tmpString))[total] = tmp[0];
-          total++;
+        numrcv = recv((*this).dstSocket, buf + total, size - total, MSG_WAITALL);
+        if (numrcv < 0) {
+          break;
+        } else if (numrcv == 0 && total < size) {
+          usleep(1000);
+          continue;
         }
-      } while (0 <= numrcv && total < size);
+        total += numrcv;
+      } while (0 < size - total);
+      buf[total] = '\0';
+      buf[size] = '\0';
+      if (0 <= total) {
+        (*((*this).tmpString)).setBinary(buf, total);
+      }
+      delete [] buf;
     }
   }
   if (total < size) {
     (*this).nErrno = errno;
+    (*this).disconnect();
   }
-  (*((*this).tmpString)).useAsBinary(total);
 
   return *((*this).tmpString);
 }
@@ -546,7 +587,7 @@ String& Socket::receiveLine() {
   fd_set readOK, mask;
   int width;
   char tmp[2];
-  int numrcv;
+  long numrcv;
 
   *((*this).tmpString) = "";
   (*((*this).tmpString)).useAsText();
@@ -572,11 +613,19 @@ String& Socket::receiveLine() {
         if (tmp[0] == '\n') {
           break;
         }
+      } else if (numrcv < 0) {
+        (*this).nErrno = -1;
+        break;
+      } else {
+        usleep(100);
       }
     } while (numrcv != 0);
     if (tmp[0] != '\n') {
       (*this).nErrno = -1;
     }
+    if ((*((*this).tmpString)).len() == 0) {
+      (*this).disconnect();
+    }
     return *((*this).tmpString);
   }
 #endif
@@ -592,13 +641,18 @@ String& Socket::receiveLine() {
   if (0 < select(width, &readOK, NULL, NULL, &timeout)) {
     if (FD_ISSET((*this).dstSocket, &readOK)) {
       do {
-        numrcv = recv((*this).dstSocket, tmp, 1, 0);
+        numrcv = recv((*this).dstSocket, tmp, 1, MSG_WAITALL);
         if (0 < numrcv) {
           tmp[1] = '\0';
           (*((*this).tmpString)) += tmp;
           if (tmp[0] == '\n') {
             break;
           }
+        } else if (numrcv < 0) {
+          (*this).nErrno = errno;
+          break;
+        } else {
+          usleep(100);
         }
       } while (0 < numrcv);
     }
@@ -606,6 +660,9 @@ String& Socket::receiveLine() {
   if (tmp[0] != '\n') {
     (*this).nErrno = errno;
   }
+  if ((*((*this).tmpString)).len() == 0) {
+    (*this).disconnect();
+  }
 
   return *((*this).tmpString);
 }
