@@ -93,11 +93,12 @@ char* charset_convert(const char* str, const char *src_charset, const char *dest
     src_ch = STR_AUTO;
   } else if (!strncasecmp(src_charset, "ASCII", 5) || !strncasecmp(src_charset, "US-ASCII", 8)) {
     src_ch = STR_ASCII;
-  } else if (!strncasecmp(src_charset, "EUC-JP", 6) || !strncasecmp(src_charset, "EUCJP", 5)) {
+  } else if (!strncasecmp(src_charset, "EUC-JP", 6) || !strncasecmp(src_charset, "X-EUC-JP", 8) ||
+      !strncasecmp(src_charset, "EUCJP", 5)) {
     src_ch = STR_EUCJP;
   } else if (!strncasecmp(src_charset, "SHIFT_JIS", 9) || !strncasecmp(src_charset, "SHIFT-JIS", 9) ||
       !strcasecmp(src_charset, "CP932") || !strcasecmp(src_charset, "WINDOWS-932") ||
-      !strcasecmp(src_charset, "MS932")) {
+      !strcasecmp(src_charset, "MS932") || !strncasecmp(src_charset, "X-SJIS", 6)) {
     src_ch = STR_SJIS;
   } else if (!strncasecmp(src_charset, "ISO-2022-JP", 11)) {
     src_ch = STR_JIS;
@@ -119,12 +120,12 @@ char* charset_convert(const char* str, const char *src_charset, const char *dest
     }
   } else if (!strncasecmp(src_charset, "EUC-KR", 6) || !strncasecmp(src_charset, "EUCKR", 5) ||
       !strcasecmp(src_charset, "CP949") || !strcasecmp(src_charset, "WINDOWS-949") ||
-      !strcasecmp(src_charset, "MS949")) {
+      !strcasecmp(src_charset, "MS949") || !strncasecmp(src_charset, "X-EUC-KR", 8)) {
     src_ch = STR_EUCKR;
   } else if (!strncasecmp(src_charset, "GB", 2) ||
       !strncasecmp(src_charset, "EUC-CN", 6) || !strncasecmp(src_charset, "EUCCN", 5) ||
       !strcasecmp(src_charset, "CP936") || !strcasecmp(src_charset, "WINDOWS-936") ||
-      !strcasecmp(src_charset, "MS936")) {
+      !strcasecmp(src_charset, "MS936") || !strncasecmp(src_charset, "X-EUC-CN", 8)) {
     src_ch = STR_GBK;
   } else if (!strcasecmp(src_charset, "BIG5") || !strncasecmp(src_charset, "EUC-TW", 6) ||
     !strncasecmp(src_charset, "EUCTW", 5) || !strcasecmp(src_charset, "CP950") ||
@@ -152,11 +153,12 @@ char* charset_convert(const char* str, const char *src_charset, const char *dest
     dest_ch = "";
   } else if (!strncasecmp(dest_charset, "ASCII", 5) || !strncasecmp(dest_charset, "US-ASCII", 8)) {
     dest_ch = STR_ASCII;
-  } else if (!strncasecmp(dest_charset, "EUC-JP", 6) || !strncasecmp(dest_charset, "EUCJP", 5)) {
+  } else if (!strncasecmp(dest_charset, "EUC-JP", 6) || !strncasecmp(dest_charset, "X-EUC-JP", 8) ||
+      !strncasecmp(dest_charset, "EUCJP", 5)) {
     dest_ch = STR_EUCJP;
   } else if (!strncasecmp(dest_charset, "SHIFT_JIS", 9) || !strncasecmp(dest_charset, "SHIFT-JIS", 9) ||
       !strcasecmp(dest_charset, "CP932") || !strcasecmp(dest_charset, "WINDOWS-932") ||
-      !strcasecmp(dest_charset, "MS932")) {
+      !strcasecmp(dest_charset, "MS932") || !strncasecmp(dest_charset, "X-SJIS", 6)) {
     dest_ch = STR_SJIS;
   } else if (!strncasecmp(dest_charset, "ISO-2022-JP", 11)) {
     dest_ch = STR_JIS;
@@ -168,12 +170,12 @@ char* charset_convert(const char* str, const char *src_charset, const char *dest
     dest_ch = STR_UTF7;
   } else if (!strncasecmp(dest_charset, "EUC-KR", 6) || !strncasecmp(dest_charset, "EUCKR", 5) ||
       !strcasecmp(dest_charset, "CP949") || !strcasecmp(dest_charset, "WINDOWS-949") ||
-      !strcasecmp(dest_charset, "MS949")) {
+      !strcasecmp(dest_charset, "MS949") || !strncasecmp(dest_charset, "X-EUC-KR", 8)) {
     dest_ch = STR_EUCKR;
   } else if (!strncasecmp(dest_charset, "GB", 2) ||
       !strncasecmp(dest_charset, "EUC-CN", 6) || !strncasecmp(src_charset, "EUCCN", 5) ||
       !strcasecmp(dest_charset, "CP936") || !strcasecmp(dest_charset, "WINDOWS-936") ||
-      !strcasecmp(dest_charset, "MS936")) {
+      !strcasecmp(dest_charset, "MS936") || !strncasecmp(dest_charset, "X-EUC-CN", 8)) {
     dest_ch = STR_GBK;
   } else if (!strcasecmp(dest_charset, "BIG5") || !strncasecmp(dest_charset, "EUC-TW", 6) ||
     !strncasecmp(dest_charset, "EUCTW", 5) || !strcasecmp(dest_charset, "CP950") ||
@@ -740,7 +742,7 @@ char* iso8859_to_utf8(const char* str, int iso8859_num) {
       if ((unsigned char)0x80 <= (unsigned char)str[i]) {
         a0 = (*iso8859_map)[(unsigned char)str[i] - (unsigned char)0x80][0];
         a1 = (*iso8859_map)[(unsigned char)str[i] - (unsigned char)0x80][1];
-	if (a0 == (unsigned char)0x00) {
+	if (a0 == (unsigned char)0x00 && (a1 & (unsigned char)0x80) == (unsigned char)0x00) {
           buf[j++] = a1;
 	} else if (a0 <= (unsigned char)0x07) {
           buf[j++] = 0xC0 | ((a0 & 7) << 2) | (a1 >> 6);
@@ -931,7 +933,7 @@ char* europe_to_utf8(const char* str, char ch) {
       if ((unsigned char)0x80 <= (unsigned char)str[i]) {
         a0 = (*eur_map)[(unsigned char)str[i] - (unsigned char)0x80][0];
         a1 = (*eur_map)[(unsigned char)str[i] - (unsigned char)0x80][1];
-	if (a0 == (unsigned char)0x00) {
+	if (a0 == (unsigned char)0x00 && (a1 & (unsigned char)0x80) == (unsigned char)0x00) {
           buf[j++] = a1;
 	} else if (a0 <= (unsigned char)0x07) {
           buf[j++] = 0xC0 | ((a0 & 7) << 2) | (a1 >> 6);
@@ -1563,7 +1565,7 @@ char* sjis_to_utf8(const char *str) {
         if (0 <= min && pos != old_pos && min <= max) {
           a0 = sjis_unicode[pos][1];
           a1 = sjis_unicode[pos][2];
-	  if (a0 == (unsigned char)0x00) {
+	  if (a0 == (unsigned char)0x00 && (a1 & (unsigned char)0x80) == (unsigned char)0x00) {
             buf[j++] = a1;
 	  } else if (a0 <= (unsigned char)0x07) {
             buf[j++] = 0xC0 | ((a0 & 7) << 2) | (a1 >> 6);
@@ -2019,7 +2021,7 @@ char* euckr_to_utf8(const char* str) {
         if (0 <= min && pos != old_pos && min <= max) {
           a0 = euckr_unicode[pos][1];
           a1 = euckr_unicode[pos][2];
-	  if (a0 == (unsigned char)0x00) {
+	  if (a0 == (unsigned char)0x00 && (a1 & (unsigned char)0x80) == (unsigned char)0x00) {
             buf[j++] = a1;
 	  } else if (a0 <= (unsigned char)0x07) {
             buf[j++] = 0xC0 | ((a0 & 7) << 2) | (a1 >> 6);
@@ -2199,7 +2201,7 @@ char* gbk_to_utf8(const char* str) {
           a0 = gbk_unicode[pos][0];
           a1 = gbk_unicode[pos][1];
           if (!(a0 == 0 && a1 == 0)) {
-	    if (a0 == (unsigned char)0x00) {
+	    if (a0 == (unsigned char)0x00 && (a1 & (unsigned char)0x80) == (unsigned char)0x00) {
               buf[j++] = a1;
 	    } else if (a0 <= (unsigned char)0x07) {
               buf[j++] = 0xC0 | ((a0 & 7) << 2) | (a1 >> 6);
@@ -2386,7 +2388,7 @@ char* big5_to_utf8(const char* str) {
         if (0 <= min && pos != old_pos && min <= max) {
           a0 = big5_unicode[pos][1];
           a1 = big5_unicode[pos][2];
-	  if (a0 == (unsigned char)0x00) {
+	  if (a0 == (unsigned char)0x00 && (a1 & (unsigned char)0x80) == (unsigned char)0x00) {
             buf[j++] = a1;
 	  } else if (a0 <= (unsigned char)0x07) {
             buf[j++] = 0xC0 | ((a0 & 7) << 2) | (a1 >> 6);
@@ -2684,7 +2686,7 @@ char* utf8_to_utf7(const char* str) {
     do {
       if (ucs4 >= UTF16BASE) {
         ucs4 -= UTF16BASE;
-        bitbuf = ((bitbuf << 16) | ((ucs4 >> UTF16SHIFT)) + UTF16HIGHSTART);
+        bitbuf = (((bitbuf << 16) | ((ucs4 >> UTF16SHIFT))) + UTF16HIGHSTART);
         ucs4 = (ucs4 & UTF16MASK) + UTF16LOSTART;
         utf16flag = 1;
       } else {
@@ -3053,7 +3055,7 @@ char* utf8_to_modutf7(const char* str) {
     do {
       if (ucs4 >= UTF16BASE) {
         ucs4 -= UTF16BASE;
-        bitbuf = ((bitbuf << 16) | ((ucs4 >> UTF16SHIFT)) + UTF16HIGHSTART);
+        bitbuf = (((bitbuf << 16) | ((ucs4 >> UTF16SHIFT))) + UTF16HIGHSTART);
         ucs4 = (ucs4 & UTF16MASK) + UTF16LOSTART;
         utf16flag = 1;
       } else {
